"""Models definitions"""

from django.core.exceptions import ValidationError
from django.conf import settings
from django.db import models
from django.urls import reverse
from django.core.validators import URLValidator, MaxValueValidator, MinValueValidator
from django.contrib.postgres.fields.array import ArrayField
from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet

URL_MAX_SIZE = 2000
DEFAULT_SORRY_PAGE = settings.PLUGINS_CONFIG["netbox_rps_plugin"]["default_sorry_page"]


class AuthenticationChoices(ChoiceSet):
    """Authentication choices definition class"""

    key = "Mapping.authentication"

    DEFAULT_VALUE = "none"

    CHOICES = [
        ("none", "None", "dark"),
        ("ldap", "Ldap", "blue"),
        ("ecas", "Ecas", "blue"),
    ]


class ApplyToChoices(ChoiceSet):
    """Apply to choices definition class"""

    key = "HttpHeader.applyTo"

    DEFAULT_VALUE = "request"

    CHOICES = [
        ("request", "Request", "blue"),
        ("response", "Response", "red"),
    ]


class ExtraProtocolChoices(ChoiceSet):
    """Protocol choices definition class"""

    key = "Mapping.protocol"

    WEBSOCKET_SECURE = "websocket"

    CHOICES = [
        (WEBSOCKET_SECURE, "Websocket", "blue"),
    ]


def default_protocol():
    """Return the default protocols"""
    return []


class Mapping(NetBoxModel):
    """Mapping definition class"""

    source = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Source",
        validators=[URLValidator(message="It must be a url")],
        unique=True,
    )
    target = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Target",
        validators=[URLValidator(message="It must be a url")],
    )
    authentication = models.CharField(
        max_length=30,
        choices=AuthenticationChoices,
        default=AuthenticationChoices.DEFAULT_VALUE,
        blank=False,
        verbose_name="Auth",
    )
    testingpage = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=True,
        null=True,
        validators=[URLValidator(message="It must be a url")],
    )
    webdav = models.BooleanField(
        default=False,
    )
    Comment = models.CharField(max_length=500, blank=True)
    gzip_proxied = models.BooleanField(default=False)
    keepalive_requests = models.IntegerField(
        default=1000, validators=[MinValueValidator(100), MaxValueValidator(5000)]
    )
    keepalive_timeout = models.IntegerField(
        default=75, validators=[MinValueValidator(1), MaxValueValidator(300)]
    )
    proxy_cache = models.BooleanField(default=False)
    proxy_read_timeout = models.IntegerField(
        default=60, validators=[MinValueValidator(1), MaxValueValidator(300)]
    )
    client_max_body_size = models.IntegerField(
        default=1, validators=[MinValueValidator(1), MaxValueValidator(255)]
    )
    sorry_page = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Sorry Page",
        validators=[URLValidator(message="It must be a url")],
        default=DEFAULT_SORRY_PAGE,
    )
    extra_protocols = ArrayField(
        base_field=models.CharField(max_length=32, choices=ExtraProtocolChoices),
        null=False,
        blank=True,
        verbose_name="Extra Protocols",
        default=default_protocol,
    )

    class Meta:
        ordering = ("source", "target")
        constraints = [
            models.CheckConstraint(
                check=~models.Q(source__exact=models.F("target")),
                name="%(app_label)s_%(class)s_check_target_source_url",
            )
        ]

    def __str__(self):
        return f"{self.source}"

    def get_absolute_url(self):
        """override"""
        return reverse("plugins:netbox_rps_plugin:mapping", args=[self.pk])

    def clean(self):
        """Clean model method for validation"""
        super().clean()

        if self.source == self.target:
            raise ValidationError(
                {"target": "Target URL cannot be equal to source URL."}
            )


class SamlConfig(NetBoxModel):
    """SAML config definition class"""

    acs_url = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="ACS URL",
        validators=[URLValidator(message="It must be a url")],
    )
    logout_url = models.CharField(
        max_length=URL_MAX_SIZE,
        blank=False,
        verbose_name="Logout URL",
        validators=[URLValidator(message="It must be a url")],
    )
    force_nauth = models.BooleanField(
        default=False,
    )
    Mapping = models.OneToOneField(
        Mapping,
        on_delete=models.CASCADE,
        related_name="saml_config",
        db_column="mapping_id",
        name="mapping",
    )


class HttpHeader(NetBoxModel):
    """HTTP Header definition class"""

    mapping = models.ForeignKey(
        Mapping, on_delete=models.CASCADE, related_name="http_headers"
    )
    name = models.CharField(max_length=120, blank=False, verbose_name="Header name")
    value = models.CharField(
        max_length=256, null=True, blank=True, verbose_name="Header value"
    )
    apply_to = models.CharField(
        max_length=30,
        choices=ApplyToChoices,
        default=ApplyToChoices.DEFAULT_VALUE,
        blank=False,
        verbose_name="Apply to",
    )

    class Meta:
        unique_together = ["mapping", "name", "apply_to"]
        ordering = ["name"]

    def __str__(self):
        return f"{self.name}"

    def get_absolute_url(self):
        """override"""
        return reverse("plugins:netbox_rps_plugin:httpheader", args=[self.pk])
